### my_plot_BioGeoBEARS_results function
# Sarah-Sophie Weil, 03/12/2020
# the original plot_BioGeoBEARS_results function cannot deal with the results 
# from trait-dependent models. 
# So I've adapted it here. Outputs are plots: legend, ancestral ranges, and 
# ancestral trait states
# (you can define which ones are produced when you call the function)


# Syntax:
# my_plot_BioGeoBEARS_results(results_object, trait_dep = FALSE,
#                             output_plot = "ances_ranges", analysis_titletxt=NULL,
#                             addl_params=list(), plotwhat="text", label.offset=NULL,
#                             tipcex=0.8, statecex=0.7, titlecex=0.8,
#                             plotlegend=FALSE, legend_ncol=NULL, legend_cex=1, 
#                             include_null_range=TRUE, tr=NULL,
#                             tipranges=NULL, color_list_for_states = NULL)

# results_object is the output from a BioGeoBEARS run, can be trait-dependent or not
# trait_dep: needs to be TRUE or FALSE. This parameter needs to be given
# output_plot: can be ances_ranges or ances_traits 
# addl_params: free parameters of the model are automatically 
# written to the title of the output, you can add some here if you like
# plotwhat: can be "text" or "pie".
# tr: tree that you used in the BioGeoBEARS analysis
# tipranges: tipranges that you used in the BioGeoBEARS analysis
# color_list_for_states: you can define the colour palette by hand
# there are defaults for all cex options if you don't include them:
# tipcex = 0.8, statecex = 0.7, titlecex = 0.8, legend_cex = 1
# default for legen_ncol: 1

## Important: if you have manually changed the states list before running BioGeoBEARS, 
## you need to do this before the plot:
# resDECx$inputs$states_list <- resDECx$inputs$lists_of_states_lists_0based[[1]]



# the original code is here: https://rdrr.io/cran/BioGeoBEARS/src/R/BioGeoBEARS_plots_v1.R

my_plot_BioGeoBEARS_results <- function(results_object, trait_dep = FALSE, 
                                        output_plot = "ances_ranges", 
                                        analysis_titletxt = NULL, 
                                        addl_params = list(), plotwhat = "text", 
                                        label.offset = NULL, tipcex = NULL, statecex = NULL, 
                                        titlecex = NULL, plotlegend = FALSE, 
                                        legend_ncol = NULL, legend_cex = NULL, 
                                        scalecex = NULL, axiscex = NULL,
                                        include_null_range = TRUE, tr = NULL, 
                                        tipranges = NULL, colors_list_for_states = NULL)
  {




# define all cex things if they haven't been defined
  if(is.null(scalecex)){
    axiscex <- 0.8
  }
  if(is.null(scalecex)){
    scalecex <- 0.8
  }
  if(is.null(tipcex)){
    tipcex <- 0.8
  }
  if(is.null(statecex)){
    statecex <- 0.7
  }
  if(is.null(titlecex)){
    titlecex <- 0.8
  }
  if(is.null(legend_cex)){
    legend_cex <- 1
  }
  

tr2 <-tr
tr2 <- reorder(tr, "pruningwise")
# Basic tree info
tips <- 1:length(tr2$tip.label)
nodes <- (length(tr2$tip.label)+1):(length(tr2$tip.label)+tr2$Nnode)


# Basic areas info
areas <- getareas_from_tipranges_object(tipranges)
areas

numareas <- length(areas)
numareas

if (!is.na(results_object$inputs$max_range_size))
{
  max_range_size = results_object$inputs$max_range_size
} else {
  max_range_size = length(areas)
}
max_range_size


if (is.null(results_object$inputs$states_list))
{
  # numstates <- numstates_from_numareas(numareas = length(areas), maxareas = max_range_size, 
  #                                      include_null_range = include_null_range)
  # numstates
  states_list <- areas_list_to_states_list_new(areas, maxareas = max_range_size, 
                                               include_null_range = include_null_range)
  #states_list
  states_list_0based_index <- 
    rcpp_areas_list_to_states_list(areas, maxareas = max_range_size, 
                                   include_null_range = include_null_range)
  #states_list_0based_index
} else {
  states_list_0based_index <- results_object$inputs$states_list
  states_list <- 
    states_list_indexes_to_areastxt(states_list = states_list_0based_index, 
                                    areanames = areas, counting_base = 0, 
                                    concat = FALSE, sep="")
}


# calculate the ML marginal probs of states at the base of each branch
# above each split (local, non-joint probs, global model)
results_object <- get_MLsplitprobs_from_results(results_object)
names(results_object)

LnL <- round(as.numeric(results_object$optim_result$value), digits=1)
LnL

params_table <- results_object$outputs@params_table
params_table

# get_perEvent_probs <- function(params_table, sumval = 1, plotwhat = "est")
# {
#   j = params_table["j", plotwhat]
#   y = params_table["y", plotwhat]
#   s = params_table["s", plotwhat]
#   v = params_table["v", plotwhat]
#   
#   jysv_val = j+y+s+v
#   
#   j = j / jysv_val * sumval
#   y = y / jysv_val * sumval
#   s = s / jysv_val * sumval
#   v = v / jysv_val * sumval
#   
#   vals = c(j, y, s, v)
#   tmpmat = matrix(vals, nrow=1)
#   wts = adf2(tmpmat)
#   names(wts) = c("j", "y", "s", "v")
#   
#   return(wts)
# }
# 
# get_perEvent_probs(params_table)


# PLOT TITLE
# What should be on the plot title

params_free_TF = params_table$type == "free"
params_free = (rownames(params_table))[params_free_TF]
numparams = sum(params_free_TF)

# Add additional user-specified parameters, if desired
if (length(addl_params) > 0)
{
  params_free <- c(params_free, unlist(addl_params))
  params_free <- unique(params_free)
}

# Write the string of FREE parameters
paramstrs <- rep("", length(params_free))
param_names <- NULL
param_ests <- NULL
for (i in 1:length(params_free))
{
  param_name <- params_free[i]
  param_est <- params_table[param_name,"est"]
  param_print <- round(param_est, digits=3)
  paramstrs[i] <- paste(param_name, "=", param_print, "; ", sep="")
  
  # Store for output
  param_names <- c(param_names, param_name)
  param_ests <- c(param_ests, param_est)
}
paramstrs <- c(paramstrs, "LnL=", LnL)
paramstr <- paste0(paramstrs, collapse="")
paramstr

# Store for output
param_names <- c("LnL", "numparams", param_names)
param_ests <- c(LnL, numparams, param_ests)

# Major title (short description)
if (is.null(analysis_titletxt))
{
  tmptxt = results_object$inputs$description
  if (any(is.null(tmptxt), tmptxt=="", tmptxt=="defaults", tmptxt=="default"))
  {
    analysis_titletxt = ""
  } else {
    analysis_titletxt = results_object$inputs$description
  }
}

analysis_titletxt = paste(analysis_titletxt, "\n", "anstates: global optim, ",
                          max_range_size, " areas max. ", paramstr, sep="")
analysis_titletxt




#######################################################
# Get the marginal probs of the splits (global ML probs, not local)
# (These are marginal, rather than joint probs; but not local optima)
#######################################################
# get_leftright_nodes_matrix_from_results <- function(tr, results_object, nodes)
# {
#   #######################################################
#   # Get the marginal probs of the splits (global ML probs, not local)
#   # (These are marginal, rather than joint probs; but not local optima)
#   #######################################################
#   
#   # Make a splits table
#   tr_table = prt(tr2, printflag=FALSE)
#   daughter_nds = tr_table$daughter_nds
#   daughter_nds = daughter_nds[nodes]
#   daughter_nds
#   leftright_nodes_matrix = matrix(data=unlist(daughter_nds), ncol=2, byrow=TRUE)
#   leftright_nodes_matrix = as.data.frame(leftright_nodes_matrix)
#   names(leftright_nodes_matrix) = c("right", "left")
#   
#   return(leftright_nodes_matrix)
# }
# leftright_nodes_matrix = get_leftright_nodes_matrix_from_results(tr2)
# 
# # This gets you the prob. of each state at the left base above each node, and
# # at the right base above each node
# marprobs = results_object$ML_marginal_prob_each_state_at_branch_bottom_below_node
# left_ML_marginals_by_node = marprobs[leftright_nodes_matrix[, 2], ]
# right_ML_marginals_by_node = marprobs[leftright_nodes_matrix[, 1], ]
# right_ML_marginals_by_node




#######################################################
# Extract the outputs ancestral states at nodes, and plot!
#######################################################
relprobs_matrix <- results_object$ML_marginal_prob_each_state_at_branch_top_AT_node

# so maybe I figured out why the plot doesn't work.... 
# the size of the relprobs_matrix is doubled for 
# trait dependent models. My hypothesis is that the first half is 
# the range states with trait in state 1
# and the second half is the range states with trait in state 2.
# So to be able to plot properly the ancestral range states, 
# I need to combine the two submatrices
if(trait_dep == TRUE){
  # calculate first probabilities of ancestral trait states
  ances_traits_mat <- matrix(NA, nrow = nrow(relprobs_matrix), ncol = 2)
  ances_traits_mat[,1] <- rowSums(relprobs_matrix[,1:(dim(relprobs_matrix)[2]/2)])
  ances_traits_mat[,2] <- rowSums(relprobs_matrix[,(dim(relprobs_matrix)[2]/2+1):
                                                    dim(relprobs_matrix)[2]])
  relprobs_matrix_for_internal_states_traits <- ances_traits_mat[nodes,]	
  # subset to just internal nodes
  
  # now calculate probabilities of ancestral range states
  relprobs_matrix <- relprobs_matrix[,1:(dim(relprobs_matrix)[2]/2)] + 
    relprobs_matrix[,(dim(relprobs_matrix)[2]/2+1):dim(relprobs_matrix)[2]]
  dim(relprobs_matrix)
  table(rowSums(relprobs_matrix))
}
relprobs_matrix_for_internal_states <- relprobs_matrix[nodes,]	
# subset to just internal nodes
statenames <- areas_list_to_states_list_new(areas, maxareas=max_range_size, 
                                            include_null_range=include_null_range, 
                                            split_ABC=FALSE)
statenames


# MLprobs <- get_ML_probs(relprobs_matrix)
# MLprobs
if(output_plot == "ances_ranges"){
  MLstates <- get_ML_states_from_relprobs(relprobs_matrix, statenames, 
                                          returnwhat = "states", if_ties = "takefirst")
} else if(output_plot == "ances_traits"){
  MLstates <- get_ML_states_from_relprobs(ances_traits_mat, c("1", "2"),
                                          returnwhat = "states", if_ties = "takefirst")
}



# Set up colors
if(is.null(colors_list_for_states)){
  # if you're plotting the traits, you need only two colours, 
  # otherwise you need as many as there are ranges
  if(output_plot == "ances_ranges"){
    colors_matrix <- get_colors_for_numareas(length(areas))
    colors_list_for_states <- mix_colors_for_states(colors_matrix, 
                                                    states_list_0based_index, 
                                                    plot_null_range = TRUE)
  } else if(output_plot == "ances_traits"){
    colors_list_for_states <- c("red", "blue")
  }
  
}



#possible_ranges_list_txt = areas_list_to_states_list_new(areas,  
#maxareas=max_range_size, split_ABC=FALSE, include_null_range=include_null_range)
# equivalent: lapply(1:length(states_list), 
# FUN = function(x){paste(states_list[[x]], collapse = "")})
if(output_plot == "ances_ranges"){
  tmp <- lapply(states_list, FUN = function(x){paste(x, collapse = "")})
  possible_ranges_list_txt <- unlist(tmp)
  cols_byNode <- rangestxt_to_colors(possible_ranges_list_txt, 
                                     colors_list_for_states, MLstates)
} else if(output_plot == "ances_traits"){
  possible_ranges_list_txt <- c(1,2)
  cols_byNode <- rangestxt_to_colors(possible_ranges_list_txt, 
                                     colors_list_for_states, MLstates)
}




# # Legend, if desired
if (plotlegend == TRUE)
{
  colors_legend(possible_ranges_list_txt, colors_list_for_states, 
                legend_ncol = legend_ncol, legend_cex = legend_cex)
}

# Plot to screen
if (is.null(label.offset))
{
  label.offset = 0.007 * get_max_height_tree(tr)
}
xlims <- c(0, 1.25*get_max_height_tree(tr))
plot(tr2, x.lim = xlims, show.tip.label = FALSE, label.offset = label.offset, 
     cex = tipcex, no.margin = FALSE)
tiplabels_to_plot = sapply(X = tr2$tip.label, FUN = substr, start = 1, stop = 30)
tiplabels(text = tiplabels_to_plot, tip = tips, cex = tipcex, adj = 0, 
          bg = "white", frame = "n", pos = 4, offset = statecex)	
# pos=4 means labels go to the right of coords

axisPhylo(cex.axis = axiscex)
mtext(text = "Millions of years ago", side = 1, line = 2, cex = scalecex)

if(output_plot == "ances_ranges"){
  # Add states / piecharts
  if (plotwhat == "text")
  {
    nodelabels(text = MLstates[nodes], node = nodes, bg = cols_byNode[nodes],
               cex = statecex)
  }
  if (plotwhat == "pie")
  {
    nodelabels(pie = relprobs_matrix_for_internal_states, node = nodes, 
               piecol = colors_list_for_states, cex = statecex)
  }
  tiplabels(text = MLstates[tips], tip = tips, bg = cols_byNode[tips], 
            cex = statecex)	# plot tiplabels either way
} else if(output_plot == "ances_traits"){
  if (plotwhat == "text")
  {
    nodelabels(text = MLstates[nodes], node = nodes, bg = cols_byNode[nodes],
               cex = statecex)
  }
  if (plotwhat == "pie")
  {
    nodelabels(pie = relprobs_matrix_for_internal_states_traits, 
               node = nodes, piecol = colors_list_for_states, cex = statecex)
  }
  tiplabels(text = MLstates[tips], tip = tips, bg = cols_byNode[tips], 
            cex = statecex)	# plot tiplabels either way
}

title(analysis_titletxt, cex.main = titlecex)


# if (plotsplits == TRUE)
# {
#   # Error check; users must specify the location of the function "plot_phylo3_nodecoords"
#   if (cornercoords_loc == "manual")
#   {
#     stoptxt = cat("\nNOTE: To plot splits, this function needs to access the function 'plot_phylo3_nodecoords'.\n",
#                   "The function is modified from an APE function, and cannot be directly included in the package,\n",
#                   "due to some C code that does not meet CRAN standards. To solve this, give plot_BioGeoBEARS_results\n",
#                   "a 'cornercoords_loc' string that gives the directory of plot_phylo3_nodecoords.R.  Typically this\n",
#                   "can be found via: ", 'tmp=np(system.file("extdata/a_scripts", package="BioGeoBEARS"))\n',
#                   "then: list.files(tmp); print(tmp)\n", sep="")
#     plotsplits = FALSE
#   }
# }
# 
# if (plotsplits == TRUE)
# {
#   #######################################################
#   # Also add the splits to the plot
#   #######################################################
#   # First, get the corner coordinates
#   coords = corner_coords(tr, tmplocation=cornercoords_loc)
#   coords
#   
#   # LEFT SPLITS
#   relprobs_matrix = left_ML_marginals_by_node
#   
#   if (plotwhat == "text")
#   {
#     MLprobs = get_ML_probs(relprobs_matrix)
#     MLprobs
#     MLstates = get_ML_states_from_relprobs(relprobs_matrix, statenames, returnwhat="states", if_ties="takefirst")
#     MLstates
#     length(MLstates)
#     
#     # Set up colors
#     possible_ranges_list_txt = areas_list_to_states_list_new(areas,  maxareas=max_range_size, split_ABC=FALSE, include_null_range=include_null_range)
#     cols_byNode = rangestxt_to_colors(possible_ranges_list_txt, colors_list_for_states, MLstates)
#     cornerlabels(text=MLstates, coords=coords$leftcorns, bg=cols_byNode, cex=splitcex)
#   }
#   
#   if (plotwhat == "pie")
#   {
#     cornerpies(pievals=relprobs_matrix, coords$leftcorns, piecol=colors_list_for_states, cex=splitcex)
#   }
#   
#   
#   
#   # RIGHT SPLITS
#   relprobs_matrix = right_ML_marginals_by_node
#   
#   if (plotwhat == "text")
#   {
#     MLprobs = get_ML_probs(relprobs_matrix)
#     MLprobs
#     MLstates = get_ML_states_from_relprobs(relprobs_matrix, statenames, returnwhat="states", if_ties="takefirst")
#     MLstates
#     length(MLstates)
#     
#     # Set up colors
#     possible_ranges_list_txt = areas_list_to_states_list_new(areas,  maxareas=max_range_size, split_ABC=FALSE, include_null_range=include_null_range)
#     cols_byNode = rangestxt_to_colors(possible_ranges_list_txt, colors_list_for_states, MLstates)
#     
#     cornerlabels(text=MLstates, coords=coords$rightcorns, bg=cols_byNode, cex=splitcex)
#   }
#   
#   if (plotwhat == "pie")
#   {
#     cornerpies(pievals=relprobs_matrix, coords$rightcorns, piecol=colors_list_for_states, cex=splitcex)			
#   }
# }
# 
# 

# Handy summary outputs
# param_ests = matrix(data=param_ests, nrow=1)
# param_ests = adf2(param_ests)
# names(param_ests) = param_names
# 
# param_ests = dfnums_to_numeric(param_ests)

#return(param_ests)
}




