## The actual BioGeoBEARS code, sourced in Script5_1 
# Sarah-Sophie Weil
# 06/01/2020


## what you need:
# - to run Script5_1 before without removing the workspace when changing the script


#### Set up BioGeoBEARS_run_objects ----

BioGeoBEARS_run_object <- define_BioGeoBEARS_run()
BioGeoBEARS_run_object$trfn <- trfn
BioGeoBEARS_run_object$geogfn <- geogfn
BioGeoBEARS_run_object$max_range_size <- max_range_size
BioGeoBEARS_run_object$min_branchlength <- 0.000001    
# Min to treat tip as a direct ancestor (no speciation event)
BioGeoBEARS_run_object$include_null_range <- TRUE   
# set to FALSE for e.g. DEC* model, DEC*+J, etc.

# Set up a time-stratified analysis:
BioGeoBEARS_run_object$timesfn <- timesfn
if(exists("dispersal_multipliers_fn")){
  BioGeoBEARS_run_object$dispersal_multipliers_fn <- dispersal_multipliers_fn
}
BioGeoBEARS_run_object$areas_allowed_fn <- areas_allowed_fn

# Speed options and multicore processing if desired
BioGeoBEARS_run_object$speedup <- FALSE          
# shorcuts to speed ML search; use FALSE if worried (e.g. >3 params)
BioGeoBEARS_run_object$use_optimx <- "GenSA"     
# if FALSE, use optim() instead of optimx(); use "GenSA" for complex analysis (>4 params)
BioGeoBEARS_run_object$num_cores_to_use <- core_parallel
BioGeoBEARS_run_object$print_optim <- TRUE
BioGeoBEARS_run_object$force_sparse <- FALSE   
# force_sparse=TRUE causes pathology & isn't much faster at this scale

# adding "fossils" -> constraining root node 
if(root_constraint == "Yes"){
  BioGeoBEARS_run_object$fixnode <- root_node
  BioGeoBEARS_run_object$fixlikes <- relprobs_matrix_for_internal_states
}

BioGeoBEARS_run_object <- readfiles_BioGeoBEARS_run(BioGeoBEARS_run_object)
BioGeoBEARS_run_object <- section_the_tree(inputs = BioGeoBEARS_run_object,
                                           make_master_table = TRUE, plot_pieces = FALSE)

# Good default settings to get ancestral states
BioGeoBEARS_run_object$return_condlikes_table <- TRUE
BioGeoBEARS_run_object$calc_TTL_loglike_from_condlikes_table <- TRUE
BioGeoBEARS_run_object$calc_ancprobs <- TRUE    
# get ancestral states from optim run

# Set up BAYAREALIKE model
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["s", "type"] <- "fixed"
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["s", "init"] <- 0.0
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["s", "est"] <- 0.0

BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["v", "type"] <- "fixed"
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["v", "init"] <- 0.0
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["v", "est"] <- 0.0

BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["ysv", "type"] <- "1-j"
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["ys", "type"] <- "ysv*1/1"
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["y", "type"] <- "1-j"

BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["mx01y", "type"] <- "fixed"
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["mx01y", "init"] <- 0.9999
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["mx01y", "est"] <- 0.9999

# this is the manually defined state space (see main script 5_1, doing this for all time slices)
for(i in 1:5){
  BioGeoBEARS_run_object$lists_of_states_lists_0based[[i]] <- states_list_0based_NEW 
}
rm(i)

# altogether there will be 8 different analyses -> initialise objects
BioGeoBEARS_run_object_BAY <- BioGeoBEARS_run_object_BAYj <- 
  BioGeoBEARS_run_object_BAYx <- BioGeoBEARS_run_object_BAYjx <-
  BioGeoBEARS_run_object_BAYm2 <- BioGeoBEARS_run_object_BAYm2j <- 
  BioGeoBEARS_run_object_BAYm2x <- BioGeoBEARS_run_object_BAYm2jx <-
  BioGeoBEARS_run_object
rm(BioGeoBEARS_run_object)





#### Traits-only model ----

# Ignoring geography (using a 1area geography), and setting most parameters to 0 or 1 
# to just estimate t12 and t21
# The loglikelihood of this model will be added later on to other models to 
# get likelihood for trait-independent dispersal
BioGeoBEARS_run_object = define_BioGeoBEARS_run()
BioGeoBEARS_run_object$print_optim = TRUE
BioGeoBEARS_run_object$max_range_size = 1
BioGeoBEARS_run_object$num_cores_to_use = 1
BioGeoBEARS_run_object$use_optimx = "GenSA"
BioGeoBEARS_run_object$speedup = TRUE
BioGeoBEARS_run_object$geogfn = paste(output_data, "/Geographies/", clade, "/", 
                                      clade, "_geography1area.data", sep = "")
BioGeoBEARS_run_object$trfn = trfn
BioGeoBEARS_run_object = readfiles_BioGeoBEARS_run(BioGeoBEARS_run_object)
BioGeoBEARS_run_object$return_condlikes_table = TRUE
BioGeoBEARS_run_object$calc_TTL_loglike_from_condlikes_table = TRUE
BioGeoBEARS_run_object$calc_ancprobs = TRUE
BioGeoBEARS_run_object$on_NaN_error = -1000000
BioGeoBEARS_run_object$force_sparse = FALSE  # works with kexpmv, but compare to dense,
# constraining root node
if(root_constraint == "Yes"){
  BioGeoBEARS_run_object$fixnode <- root_node
  # since it is now a 1 area geography, I need to slightly transform the relprobs_matrix
  
  relprobs_matrix_for_internal_states <- 
    cbind(0, 
          rowSums(relprobs_matrix_for_internal_states[
            ,1:(dim(relprobs_matrix_for_internal_states)[2]/2)]),
          0, rowSums(relprobs_matrix_for_internal_states[
            ,(dim(relprobs_matrix_for_internal_states)[2]/2+1):
              dim(relprobs_matrix_for_internal_states)[2]]) )
  
  print(relprobs_matrix_for_internal_states)
  BioGeoBEARS_run_object$fixlikes <- relprobs_matrix_for_internal_states
}



BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["a","type"] = "fixed"
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["a","init"] = 0.0
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["a","est"] = 0.0

BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["d","type"] = "fixed"
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["d","init"] = 0.0
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["d","est"] = 0.0

BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["e","type"] = "fixed"
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["e","init"] = 0.0
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["e","est"] = 0.0

# Set up BAYAREALIKE model
# No subset sympatry
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["s","type"] = "fixed"
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["s","init"] = 0.0
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["s","est"] = 0.0

# No vicariance
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["v","type"] = "fixed"
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["v","init"] = 0.0
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["v","est"] = 0.0

# Adjust linkage between parameters
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["ysv","type"] = "1-j"
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["ys","type"] = "ysv*1/1"
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["y","type"] = "1-j"

# Only sympatric/range-copying (y) events allowed, and with 
# exact copying (both descendants always the same size as the ancestor)
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["mx01y","type"] = "fixed"
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["mx01y","init"] = 0.9999
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["mx01y","est"] = 0.9999

# Add the traits data and model
BioGeoBEARS_run_object <- add_trait_to_BioGeoBEARS_run_object(BioGeoBEARS_run_object,
                                                              trait_fn = trait_fn)

# BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table


# Manual modifications of trait-based model
# Edit t12 and t21 rates
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["t12", "type"] = "free"
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["t12", "init"] = 0.001
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["t12", "est"] = 0.001
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["t12", "min"] = 0.00001
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["t12", "max"] = 1

BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["t21", "type"] = "free"
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["t21", "init"] = 0.001
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["t21", "est"] = 0.001
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["t21", "min"] = 0.00001
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["t21", "max"] = 1

# No multipliers on geog (set m1 and m2 to 1)
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["m1", "type"] = "fixed"
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["m2", "type"] = "fixed"
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["m1", "init"] = 1.0
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["m2", "init"] = 1.0
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["m1", "est"] = 1.0
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["m2", "est"] = 1.0
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["m1", "desc"] = 
  "trait-based dispersal rate multipliers m1"
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["m2", "desc"] = 
  "trait-based dispersal rate multipliers m2"

# Run this to check inputs. Read the error messages if you get them!
BioGeoBEARS_run_object <- fix_BioGeoBEARS_params_minmax(BioGeoBEARS_run_object)
check_BioGeoBEARS_run(BioGeoBEARS_run_object)

print("Printing warnings: 'warnings()':")
print(warnings())

resfn <- sub(".Rdata", paste0("_", traits_incl, ".Rdata"), resfn_all[[3]][1])
if (runslow){
  res <- bears_optim_run(BioGeoBEARS_run_object)
  res
  save(res, file = resfn)
  resTraits = res
} else {
  # Loads to "res"
  load(resfn)
  resTraits <- res
} 
rm(BioGeoBEARS_run_object)





#### BAYAREALIKE ----
resfn <- resfn_all[[3]][2]
load(resfn)
resBAY <- res
rm(BioGeoBEARS_run_object_BAY)







#### BAYAREALIKE+m2 ----
BioGeoBEARS_run_object_BAYm2$speedup <- FALSE
BioGeoBEARS_run_object_BAYm2 <- 
  add_trait_to_BioGeoBEARS_run_object(BioGeoBEARS_run_object_BAYm2, trait_fn = trait_fn)

# using just trait model and BAY as starting points
t12_start <- resTraits$outputs@params_table["t12","est"]
t21_start <- resTraits$outputs@params_table["t21","est"]
dstart <- resBAY$outputs@params_table["d", "est"]
estart <- resBAY$outputs@params_table["e", "est"]
m2_start <- 1

BioGeoBEARS_run_object_BAYm2$BioGeoBEARS_model_object@params_table["d", "init"] <- dstart
BioGeoBEARS_run_object_BAYm2$BioGeoBEARS_model_object@params_table["d", "est"] <- dstart
BioGeoBEARS_run_object_BAYm2$BioGeoBEARS_model_object@params_table["e", "init"] <- estart
BioGeoBEARS_run_object_BAYm2$BioGeoBEARS_model_object@params_table["e", "est"] <- estart

# Manual modifications of trait-based model
BioGeoBEARS_run_object_BAYm2$BioGeoBEARS_model_object@params_table["t12", "type"] <- "free"
BioGeoBEARS_run_object_BAYm2$BioGeoBEARS_model_object@params_table["t12", "init"] <- t12_start
BioGeoBEARS_run_object_BAYm2$BioGeoBEARS_model_object@params_table["t12", "est"] <- t12_start
BioGeoBEARS_run_object_BAYm2$BioGeoBEARS_model_object@params_table["t12", "min"] <- 0.00001
BioGeoBEARS_run_object_BAYm2$BioGeoBEARS_model_object@params_table["t12", "max"] <- 
  round(max(t12_start, t21_start)*5, 3)

BioGeoBEARS_run_object_BAYm2$BioGeoBEARS_model_object@params_table["t21", "type"] <- "free"
BioGeoBEARS_run_object_BAYm2$BioGeoBEARS_model_object@params_table["t21", "init"] <- t21_start
BioGeoBEARS_run_object_BAYm2$BioGeoBEARS_model_object@params_table["t21", "est"] <- t21_start
BioGeoBEARS_run_object_BAYm2$BioGeoBEARS_model_object@params_table["t21", "min"] <- 0.00001
BioGeoBEARS_run_object_BAYm2$BioGeoBEARS_model_object@params_table["t21", "max"] <- 
  round(max(t12_start, t21_start)*5, 3)



BioGeoBEARS_run_object_BAYm2$BioGeoBEARS_model_object@params_table["m1", "type"] <- "fixed"
BioGeoBEARS_run_object_BAYm2$BioGeoBEARS_model_object@params_table["m1", "init"] <- 1
BioGeoBEARS_run_object_BAYm2$BioGeoBEARS_model_object@params_table["m1", "est"] <- 1
BioGeoBEARS_run_object_BAYm2$BioGeoBEARS_model_object@params_table["m1", "min"] <- 0.01
BioGeoBEARS_run_object_BAYm2$BioGeoBEARS_model_object@params_table["m1", "max"] <- 1

BioGeoBEARS_run_object_BAYm2$BioGeoBEARS_model_object@params_table["m2", "type"] <- "free"
BioGeoBEARS_run_object_BAYm2$BioGeoBEARS_model_object@params_table["m2", "init"] <- m2_start
BioGeoBEARS_run_object_BAYm2$BioGeoBEARS_model_object@params_table["m2", "est"] <- m2_start
BioGeoBEARS_run_object_BAYm2$BioGeoBEARS_model_object@params_table["m2", "min"] <- 0
BioGeoBEARS_run_object_BAYm2$BioGeoBEARS_model_object@params_table["m2", "max"] <- 10

BioGeoBEARS_run_object_BAYm2 <- fix_BioGeoBEARS_params_minmax(BioGeoBEARS_run_object_BAYm2)
check_BioGeoBEARS_run(BioGeoBEARS_run_object_BAYm2)

print("Printing warnings: 'warnings()':")
print(warnings())

resfn <- sub(".Rdata", paste0("_", traits_incl, ".Rdata"), resfn_all[[3]][6])
t6 <- proc.time()
if(runslow){
  res <- bears_optim_run(BioGeoBEARS_run_object_BAYm2)
  save(res, file = resfn)
  resBAYm2 <- res 
} else {
  load(resfn)
  resBAYm2 <- res
}
t7 <- proc.time()
rm(BioGeoBEARS_run_object_BAYm2)
t7 - t6


