## The actual BioGeoBEARS code, sourced in Script5_1 
# Sarah-Sophie Weil
# 06/01/2020


## what you need:
# - to run Script5_1 before without removing the workspace when changing the script


#### Set up BioGeoBEARS_run_objects ----

BioGeoBEARS_run_object <- define_BioGeoBEARS_run()
BioGeoBEARS_run_object$trfn <- trfn
BioGeoBEARS_run_object$geogfn <- geogfn
BioGeoBEARS_run_object$max_range_size <- max_range_size
BioGeoBEARS_run_object$min_branchlength <- 0.000001    
# Min to treat tip as a direct ancestor (no speciation event)
BioGeoBEARS_run_object$include_null_range <- TRUE    
# set to FALSE for e.g. DEC* model, DEC*+J, etc.

# Set up a time-stratified analysis:
BioGeoBEARS_run_object$timesfn <- timesfn
if(exists("dispersal_multipliers_fn")){
  BioGeoBEARS_run_object$dispersal_multipliers_fn <- dispersal_multipliers_fn
}
BioGeoBEARS_run_object$areas_allowed_fn <- areas_allowed_fn
#BioGeoBEARS_run_object$areas_adjacency_fn = "areas_adjacency.txt"

# Speed options and multicore processing if desired
BioGeoBEARS_run_object$speedup <- FALSE          
# shorcuts to speed ML search; use FALSE if worried (e.g. >3 params)
BioGeoBEARS_run_object$use_optimx <- "GenSA"     
# if FALSE, use optim() instead of optimx(); use "GenSA" for complex analysis (>4 params)
BioGeoBEARS_run_object$num_cores_to_use <- core_parallel
BioGeoBEARS_run_object$print_optim <- TRUE
BioGeoBEARS_run_object$force_sparse <- FALSE    
# force_sparse=TRUE causes pathology & isn't much faster at this scale

# adding "fossils" -> constraining root node
if(root_constraint == "Yes"){
  BioGeoBEARS_run_object$fixnode <- root_node
  BioGeoBEARS_run_object$fixlikes <- relprobs_matrix_for_internal_states
}

BioGeoBEARS_run_object <- readfiles_BioGeoBEARS_run(BioGeoBEARS_run_object)
BioGeoBEARS_run_object <- section_the_tree(inputs = BioGeoBEARS_run_object, 
                                           make_master_table = TRUE, plot_pieces = FALSE)

# Good default settings to get ancestral states
BioGeoBEARS_run_object$return_condlikes_table <- TRUE
BioGeoBEARS_run_object$calc_TTL_loglike_from_condlikes_table <- TRUE
BioGeoBEARS_run_object$calc_ancprobs <- TRUE    # get ancestral states from optim run

# Set up BAYAREALIKE model
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["s", "type"] <- "fixed"
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["s", "init"] <- 0.0
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["s", "est"] <- 0.0

BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["v", "type"] <- "fixed"
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["v", "init"] <- 0.0
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["v", "est"] <- 0.0

BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["ysv", "type"] <- "1-j"
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["ys", "type"] <- "ysv*1/1"
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["y", "type"] <- "1-j"

BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["mx01y", "type"] <- "fixed"
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["mx01y", "init"] <- 0.9999
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["mx01y", "est"] <- 0.9999

# this is the manually defined state space (see main script 5_1, doing this for all time slices)
for(i in 1:5){
  BioGeoBEARS_run_object$lists_of_states_lists_0based[[i]] <- states_list_0based_NEW 
}
rm(i)

# altogether there will be 8 different analyses -> initialise objects
BioGeoBEARS_run_object_BAY <- BioGeoBEARS_run_object_BAYj <- 
  BioGeoBEARS_run_object_BAYx <- BioGeoBEARS_run_object_BAYjx <-
  BioGeoBEARS_run_object_BAYm2 <- BioGeoBEARS_run_object_BAYm2j <- 
  BioGeoBEARS_run_object_BAYm2x <- BioGeoBEARS_run_object_BAYm2jx <-
  BioGeoBEARS_run_object
rm(BioGeoBEARS_run_object)







#### BAYAREALIKE+x ----
resfn <- resfn_all[[3]][4]
load(resfn)
resBAYx <- res
rm(BioGeoBEARS_run_object_BAYx)






#### BAYAREALIKE+m2 ----
resfn <- sub(".Rdata", paste0("_", traits_incl, ".Rdata"), resfn_all[[3]][6])
load(resfn)
resBAYm2 <- res
rm(BioGeoBEARS_run_object_BAYm2)






#### BAYAREALIKE+m2+x ----
BioGeoBEARS_run_object_BAYm2x <- 
  add_trait_to_BioGeoBEARS_run_object(BioGeoBEARS_run_object_BAYm2x, trait_fn = trait_fn)
BioGeoBEARS_run_object_BAYm2x$speedup <- FALSE
BioGeoBEARS_run_object_BAYm2x$distsfn <- geogdist_fn
BioGeoBEARS_run_object_BAYm2x <- readfiles_BioGeoBEARS_run(BioGeoBEARS_run_object_BAYm2x)

# using previous model as starting points
t12_start <- resBAYm2$outputs@params_table["t12","est"]
t21_start <- resBAYm2$outputs@params_table["t21","est"]
dstart <- resBAYm2$outputs@params_table["d", "est"]
estart <- resBAYm2$outputs@params_table["e", "est"]
m2_start <- resBAYm2$outputs@params_table["m2", "est"]
xstart <- resBAYx$outputs@params_table["x", "est"]

BioGeoBEARS_run_object_BAYm2x$BioGeoBEARS_model_object@params_table["d", "init"] <- dstart
BioGeoBEARS_run_object_BAYm2x$BioGeoBEARS_model_object@params_table["d", "est"] <- dstart
BioGeoBEARS_run_object_BAYm2x$BioGeoBEARS_model_object@params_table["e", "init"] <- estart
BioGeoBEARS_run_object_BAYm2x$BioGeoBEARS_model_object@params_table["e", "est"] <- estart

# Add x as a free parameter
BioGeoBEARS_run_object_BAYm2x$BioGeoBEARS_model_object@params_table["x","type"] = "free"
BioGeoBEARS_run_object_BAYm2x$BioGeoBEARS_model_object@params_table["x","init"] = xstart
BioGeoBEARS_run_object_BAYm2x$BioGeoBEARS_model_object@params_table["x","est"] = xstart
BioGeoBEARS_run_object_BAYm2x$BioGeoBEARS_model_object@params_table["x","max"] = -0.000001
BioGeoBEARS_run_object_BAYm2x$BioGeoBEARS_model_object@params_table["x","min"] = -5

# Manual modifications of trait-based model
BioGeoBEARS_run_object_BAYm2x$BioGeoBEARS_model_object@params_table["t12", "type"] <- "free"
BioGeoBEARS_run_object_BAYm2x$BioGeoBEARS_model_object@params_table["t12", "init"] <- t12_start
BioGeoBEARS_run_object_BAYm2x$BioGeoBEARS_model_object@params_table["t12", "est"] <- t12_start
BioGeoBEARS_run_object_BAYm2x$BioGeoBEARS_model_object@params_table["t12", "min"] <- 0.00001
BioGeoBEARS_run_object_BAYm2x$BioGeoBEARS_model_object@params_table["t12", "max"] <- 
  round(max(t12_start, t21_start)*5, 3)

BioGeoBEARS_run_object_BAYm2x$BioGeoBEARS_model_object@params_table["t21", "type"] <- "free"
BioGeoBEARS_run_object_BAYm2x$BioGeoBEARS_model_object@params_table["t21", "init"] <- t21_start
BioGeoBEARS_run_object_BAYm2x$BioGeoBEARS_model_object@params_table["t21", "est"] <- t21_start
BioGeoBEARS_run_object_BAYm2x$BioGeoBEARS_model_object@params_table["t21", "min"] <- 0.00001
BioGeoBEARS_run_object_BAYm2x$BioGeoBEARS_model_object@params_table["t21", "max"] <- 
  round(max(t12_start, t21_start)*5, 3)



BioGeoBEARS_run_object_BAYm2x$BioGeoBEARS_model_object@params_table["m1", "type"] <- "fixed"
BioGeoBEARS_run_object_BAYm2x$BioGeoBEARS_model_object@params_table["m1", "init"] <- 1
BioGeoBEARS_run_object_BAYm2x$BioGeoBEARS_model_object@params_table["m1", "est"] <- 1
BioGeoBEARS_run_object_BAYm2x$BioGeoBEARS_model_object@params_table["m1", "min"] <- 0.01
BioGeoBEARS_run_object_BAYm2x$BioGeoBEARS_model_object@params_table["m1", "max"] <- 1

BioGeoBEARS_run_object_BAYm2x$BioGeoBEARS_model_object@params_table["m2", "type"] <- "free"
BioGeoBEARS_run_object_BAYm2x$BioGeoBEARS_model_object@params_table["m2", "init"] <- m2_start
BioGeoBEARS_run_object_BAYm2x$BioGeoBEARS_model_object@params_table["m2", "est"] <- m2_start
BioGeoBEARS_run_object_BAYm2x$BioGeoBEARS_model_object@params_table["m2", "min"] <- 0
BioGeoBEARS_run_object_BAYm2x$BioGeoBEARS_model_object@params_table["m2", "max"] <- 1

BioGeoBEARS_run_object_BAYm2x <- fix_BioGeoBEARS_params_minmax(BioGeoBEARS_run_object_BAYm2x)
check_BioGeoBEARS_run(BioGeoBEARS_run_object_BAYm2x)

print("Printing warnings: 'warnings()':")
print(warnings())

resfn <- sub(".Rdata", paste0("_", traits_incl, ".Rdata"), resfn_all[[3]][8])
t1 <- proc.time()
if(runslow){
  res <- bears_optim_run(BioGeoBEARS_run_object_BAYm2x)
  save(res, file = resfn)
  resBAYm2x <- res 
} else {
  load(resfn)
  resBAYm2x <- res
}
t2 <- proc.time()
rm(BioGeoBEARS_run_object_BAYm2x)
t2 - t1
