## The actual BioGeoBEARS code, sourced in Script5_1 
# Sarah-Sophie Weil
# 06/01/2020


## what you need:
# - to run Script5_1 before without removing the workspace when changing the script



#### Set up BioGeoBEARS_run_objects ----
BioGeoBEARS_run_object <- define_BioGeoBEARS_run()
BioGeoBEARS_run_object$trfn <- trfn
BioGeoBEARS_run_object$geogfn <- geogfn
BioGeoBEARS_run_object$max_range_size <- max_range_size
BioGeoBEARS_run_object$min_branchlength <- 0.000001    
# Min to treat tip as a direct ancestor (no speciation event)
BioGeoBEARS_run_object$include_null_range <- TRUE    
# set to FALSE for e.g. DEC* model, DEC*+J, etc.

# Set up a time-stratified analysis:
BioGeoBEARS_run_object$timesfn <- timesfn
if(exists("dispersal_multipliers_fn")){
  BioGeoBEARS_run_object$dispersal_multipliers_fn <- dispersal_multipliers_fn
}
BioGeoBEARS_run_object$areas_allowed_fn <- areas_allowed_fn

# Speed options and multicore processing if desired
BioGeoBEARS_run_object$speedup <- FALSE         
# shorcuts to speed ML search; use FALSE if worried (e.g. >3 params)
BioGeoBEARS_run_object$use_optimx <- "GenSA"     
# if FALSE, use optim() instead of optimx(); use "GenSA" for complex analysis (>4 params)
BioGeoBEARS_run_object$num_cores_to_use <- core_parallel
BioGeoBEARS_run_object$print_optim <- TRUE
BioGeoBEARS_run_object$force_sparse <- FALSE    
# force_sparse=TRUE causes pathology & isn't much faster at this scale

# adding "fossils" -> constraining root node
if(root_constraint == "Yes"){
  BioGeoBEARS_run_object$fixnode <- root_node
  BioGeoBEARS_run_object$fixlikes <- relprobs_matrix_for_internal_states
}

BioGeoBEARS_run_object <- readfiles_BioGeoBEARS_run(BioGeoBEARS_run_object)
BioGeoBEARS_run_object <- section_the_tree(inputs = BioGeoBEARS_run_object,
                                           make_master_table = TRUE, plot_pieces = FALSE)

# Good default settings to get ancestral states
BioGeoBEARS_run_object$return_condlikes_table <- TRUE
BioGeoBEARS_run_object$calc_TTL_loglike_from_condlikes_table <- TRUE
BioGeoBEARS_run_object$calc_ancprobs <- TRUE    # get ancestral states from optim run

# Set up BAYAREALIKE model
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["s", "type"] <- "fixed"
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["s", "init"] <- 0.0
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["s", "est"] <- 0.0

BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["v", "type"] <- "fixed"
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["v", "init"] <- 0.0
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["v", "est"] <- 0.0

BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["ysv", "type"] <- "1-j"
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["ys", "type"] <- "ysv*1/1"
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["y", "type"] <- "1-j"

BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["mx01y", "type"] <- "fixed"
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["mx01y", "init"] <- 0.9999
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["mx01y", "est"] <- 0.9999

# this is the manually defined state space (see main script 5_1, doing this for all time slices)
for(i in 1:5){
  BioGeoBEARS_run_object$lists_of_states_lists_0based[[i]] <- states_list_0based_NEW 
}
rm(i)

# altogether there will be 8 different analyses -> initialise objects
BioGeoBEARS_run_object_BAY <- BioGeoBEARS_run_object_BAYj <-
  BioGeoBEARS_run_object_BAYx <- BioGeoBEARS_run_object_BAYjx <-
  BioGeoBEARS_run_object_BAYm2 <- BioGeoBEARS_run_object_BAYm2j <- 
  BioGeoBEARS_run_object_BAYm2x <- BioGeoBEARS_run_object_BAYm2jx <-
  BioGeoBEARS_run_object
rm(BioGeoBEARS_run_object)





#### BAYAREALIKE ----
resfn <- resfn_all[[3]][2]
load(resfn)
resBAY <- res
rm(BioGeoBEARS_run_object_BAY)







#### BAYAREALIKE+x ----
BioGeoBEARS_run_object_BAYx$speedup <- TRUE 
BioGeoBEARS_run_object_BAYx$distsfn <- geogdist_fn
BioGeoBEARS_run_object_BAYx <- readfiles_BioGeoBEARS_run(BioGeoBEARS_run_object_BAYx)

# using outputs from BAY analysis as starting points
dstart <- resBAY$outputs@params_table["d", "est"]
estart <- resBAY$outputs@params_table["e", "est"]

# Input starting values for d, e
BioGeoBEARS_run_object_BAYx$BioGeoBEARS_model_object@params_table["d", "init"] <- dstart
BioGeoBEARS_run_object_BAYx$BioGeoBEARS_model_object@params_table["d", "est"] <- dstart
BioGeoBEARS_run_object_BAYx$BioGeoBEARS_model_object@params_table["e", "init"] <- estart
BioGeoBEARS_run_object_BAYx$BioGeoBEARS_model_object@params_table["e", "est"] <- estart

# Add x as a free parameter
# the parameters are taken from the biogeobears google group
BioGeoBEARS_run_object_BAYx$BioGeoBEARS_model_object@params_table["x","type"] = "free"
BioGeoBEARS_run_object_BAYx$BioGeoBEARS_model_object@params_table["x","init"] = 0
BioGeoBEARS_run_object_BAYx$BioGeoBEARS_model_object@params_table["x","est"] = -0.000001
BioGeoBEARS_run_object_BAYx$BioGeoBEARS_model_object@params_table["x","max"] = 0
BioGeoBEARS_run_object_BAYx$BioGeoBEARS_model_object@params_table["x","min"] = -5

check_BioGeoBEARS_run(BioGeoBEARS_run_object_BAYx)

print("Printing warnings: 'warnings()':")
print(warnings())

resfn <- resfn_all[[3]][4]
t1 <- proc.time()
if(runslow){
  res <- bears_optim_run(BioGeoBEARS_run_object_BAYx)
  save(res, file = resfn)
  resBAYx <- res 
} else {
  load(resfn)
  resBAYx <- res
}
t2 <- proc.time()
rm(BioGeoBEARS_run_object_BAYx)
t2 - t1




