## The actual BioGeoBEARS code, sourced in Script5_1 
# Sarah-Sophie Weil
# 06/01/2020


## what you need:
# - to run Script5_1 before without removing the workspace when changing the script

#### Set up BioGeoBEARS_run_objects ----
BioGeoBEARS_run_object <- define_BioGeoBEARS_run()
BioGeoBEARS_run_object$trfn <- trfn
BioGeoBEARS_run_object$geogfn <- geogfn
BioGeoBEARS_run_object$max_range_size <- max_range_size
BioGeoBEARS_run_object$min_branchlength <- 0.000001   
# Min to treat tip as a direct ancestor (no speciation event)
BioGeoBEARS_run_object$include_null_range <- TRUE   
# set to FALSE for e.g. DEC* model, DEC*+J, etc.

# Set up a time-stratified analysis:
BioGeoBEARS_run_object$timesfn <- timesfn
if(exists("dispersal_multipliers_fn")){
  BioGeoBEARS_run_object$dispersal_multipliers_fn <- dispersal_multipliers_fn
}
BioGeoBEARS_run_object$areas_allowed_fn <- areas_allowed_fn

# Speed options and multicore processing if desired
BioGeoBEARS_run_object$speedup <- FALSE         
# shorcuts to speed ML search; use FALSE if worried (e.g. >3 params)
BioGeoBEARS_run_object$use_optimx <- "GenSA"     
# if FALSE, use optim() instead of optimx(); use "GenSA" for complex analysis (>4 params)
BioGeoBEARS_run_object$num_cores_to_use <- core_parallel
BioGeoBEARS_run_object$print_optim <- TRUE
BioGeoBEARS_run_object$force_sparse <- FALSE    
# force_sparse=TRUE causes pathology & isn't much faster at this scale

# adding "fossils" -> constraining root node to what was estimated in the DECm2x runs
if(root_constraint == "Yes"){
  BioGeoBEARS_run_object$fixnode <- root_node
  BioGeoBEARS_run_object$fixlikes <- relprobs_matrix_for_internal_states
}

BioGeoBEARS_run_object <- readfiles_BioGeoBEARS_run(BioGeoBEARS_run_object)
BioGeoBEARS_run_object <- section_the_tree(inputs = BioGeoBEARS_run_object, 
                                           make_master_table = TRUE, plot_pieces = FALSE)

# Good default settings to get ancestral states
BioGeoBEARS_run_object$return_condlikes_table <- TRUE
BioGeoBEARS_run_object$calc_TTL_loglike_from_condlikes_table <- TRUE
BioGeoBEARS_run_object$calc_ancprobs <- TRUE    # get ancestral states from optim run

# this is the manually defined state space (see main script 5_1, doing this for all time slices)
for(i in 1:5){
  BioGeoBEARS_run_object$lists_of_states_lists_0based[[i]] <- states_list_0based_NEW 
}
rm(i)


# altogether there will be 8 different analyses -> initialise objects
BioGeoBEARS_run_object_DEC <- BioGeoBEARS_run_object_DECj <-
  BioGeoBEARS_run_object_DECx <- BioGeoBEARS_run_object_DECjx <-
  BioGeoBEARS_run_object_DECm2 <- BioGeoBEARS_run_object_DECm2j <-
  BioGeoBEARS_run_object_DECm2x <- BioGeoBEARS_run_object_DECm2jx <-
  BioGeoBEARS_run_object
rm(BioGeoBEARS_run_object)





#### DEC ----
BioGeoBEARS_run_object_DEC$speedup <- TRUE
# Set up DEC model
check_BioGeoBEARS_run(BioGeoBEARS_run_object_DEC)

print("Printing warnings: 'warnings()':")
print(warnings())

# For a slow analysis, run once, then set runslow=FALSE to just load the saved result
resfn <- resfn_all[[1]][2]
t1 <- proc.time()
if(runslow){
  res <- bears_optim_run(BioGeoBEARS_run_object_DEC)
  # important here is the output of optimx: convcode = 0 means that convergence was successful
  if(sv.output == 1){
    save(res, file = resfn) 
  }
  resDEC <- res
} else {
  load(resfn)
  resDEC <- res
}
t2 <- proc.time()
rm(BioGeoBEARS_run_object_DEC)
t2 - t1
